@@ -295,8 +295,10 @@ extern bool formatOverflow;
 extern int webTimeout, mailTimeout;
 extern uchar browseLocal;
 extern bool sqlPresent;		/* Was edbrowse compiled with SQL built in? */
-extern bool curlActive; // is curl running?
+extern bool curlActive; // curl is running?
+extern const char *curlCiphers; // cipher string to send to curl
 extern bool ismc;		// Is the program running as a mail client?
+extern bool browseMail;		// Are we browsing a mail in html?
 extern bool passMail;		// pass mail across the filters
 extern bool attimg; // show atached images
 extern bool down_bg;		// download in background
@@ -418,7 +420,6 @@ struct ebFrame {
 	bool jslink; // linke to javascript
 	bool browseMode;
 	bool xmlMode;
-	short jtmin;
 	char *dw;		// document.write string
 	int dw_l;		// length of the above
 // document.writes go under the body.
@@ -493,6 +494,7 @@ int ehsn;
 /* Various text modes, these are incompatible with binMode */
 /* All modes convert to utf8, as that is what pcre understands. */
 	bool utf8Mode:1;
+	bool utf8Mark:1;
 	bool utf16Mode:1;
 	bool utf32Mode:1;
 	bool bigMode:1; // big-endian
