@@ -25,6 +25,7 @@ void saveSubstitutionStrings(void);
 void restoreSubstitutionStrings(Window *nw);
 Window *createWindow(void);
 void freeWindowLines(struct lineMap *map);
+void undoCompare(void);
 void freeWindows(int cx, bool all);
 bool cxCompare(int cx) ;
 bool cxActive(int cx, bool error);
@@ -42,7 +43,6 @@ bool runCommand(const char *line);
 bool edbrowseCommand(const char *line, bool script) ;
 int sideBuffer(int cx, const char *text, int textlen, const char *bufname);
 void freeEmptySideBuffer(int n);
-bool browseCurrentBuffer(const char *suffix, bool plain) ;
 bool locateTagInBuffer(int tagno, int *ln1_p, int *ln2_p, char **p1_p, char **p2_p, char **s_p, char **t_p);
 char *getFieldFromBuffer(int tagno, int ln0);
 int fieldIsChecked(int tagno);
@@ -92,6 +92,7 @@ bool rfWhileReading();
 bool addFolders();
 bool deleteFolder(int ln);
 bool renameFolder(const char *src, const char *dest);
+void imapCleanupInBackground(CURL *h);
 
 // sourcefile=format.c
 void prepareForBrowse(char *h, int h_len);
@@ -139,6 +140,8 @@ void showErrorConditional(char cmd);
 void showErrorAbort(void);
 void eb_puts(const char *s);
 bool helpUtility(void);
+bool varCommand(const char *line);
+bool varExpand(const char *line, char **newline);
 
 // sourcefile=html.c
 void dwStart(void);
@@ -153,6 +156,7 @@ void runScriptsPending(bool startbrowse);
 void preFormatCheck(int tagno, bool * pretag, bool * slash);
 char *htmlParse(char *buf, int remote);
 bool htmlTest(void);
+bool browseCurrentBuffer(const char *suffix, bool plain) ;
 void infShow(int tagno, const char *search);
 bool infReplace(int tagno, char *newtext, bool notify);
 char *displayOptions(const Tag *sel);
@@ -290,6 +294,7 @@ bool stringIsDate(const char *s) ;
 bool stringIsFloat(const char *s, double *dp) ;
 bool memEqualCI(const char *s, const char *t, int len) ;
 const char *stringInBufLine(const char *s, const char *t);
+const char *charInBufLine(const char *s, char t);
 bool stringEqual(const char *s, const char *t) ;
 bool stringEqualCI(const char *s, const char *t) ;
 int stringInList(const char *const *list, const char *s) ;
@@ -315,7 +320,7 @@ void copyPstring(pst s, const pst t) ;
 int comparePstring(const uchar * s, const uchar * t);
 int fdIntoMemory(int fd, char **data, int *len, bool inparts);
 int fileIntoMemory(const char *filename, char **data, int *len, bool inparts);
-bool memoryOutToFile(const char *filename, const char *data, int len, int msgcreate, int msgwrite) ;
+bool memoryOutToFile(const char *filename, const char *data, int len);
 void truncate0(const char *filename, int fh);
 long long bufferSizeW(const Window *w, bool browsing);
 long long bufferSize(int cx, bool browsing);
@@ -408,6 +413,7 @@ bool getUserPassRealm(const char *url, char *creds, const char *realm);
 // atomic operation, so the previous routines can run from other threads.
 bool addWebAuthorization(const char *url, const char *credentials, bool proxy, const char *realm);
 const char *edbrowseTempFilename(const char *suffix, bool output);
+char *file2suffix(const char *filename);
 const struct MIMETYPE *findMimeByURL(const char *url, uchar *sxfirst);
 const struct MIMETYPE *findMimeBySuffix(const char *suffix);
 const struct MIMETYPE *findMimeByFile(const char *filename);
