@@ -1,6 +1,6 @@
 // format.c, Format text, establish line breaks, manage whitespace,
 // iso8859, utf8, utf16, utf32, base64, color codes,
-// emojis, messages in your local language.
+// emojis, messages in your local language, edbrowse variables.
 
 #include "eb.h"
 
@@ -286,7 +286,7 @@ static void anchorSwap(char *buf)
 	bool slash;		// closing tag
 	bool change;		// made a swap somewhere
 	bool strong;		// strong whitespace, newline or paragraph
-	int n, cnt, tagno;
+	int n = 0, cnt, tagno = 0;
 	char tag[20];
 
 	cnt = 0;
@@ -1742,7 +1742,8 @@ void utfHigh(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p,
 		return;
 	}
 
-	outbuf = allocMem(inbuflen * 4);	// worst case
+// worst case is utf32
+	outbuf = allocMem(inbuflen * 4);
 
 	i = j = 0;
 	while (i < inbuflen) {
@@ -1782,6 +1783,7 @@ void utfHigh(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p,
 			}
 			continue;
 		}
+
 // utf16, a bit trickier but not too bad.
 		if (unicode <= 0xd7ff
 		    || (unicode >= 0xe000 && unicode <= 0xffff)) {
@@ -1984,42 +1986,17 @@ void utfLow(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p,
 // This is called by readFile in buffers.c, so has some
 void diagnoseAndConvert (char **rbuf_p, bool *isAllocated_p, int *partSize_p, const bool firstPart, const bool showMessage)
 {
+	if (!iuConvert) return;
+
 	char *rbuf = *rbuf_p;
 	char *tbuf;
 	int i, j;
 	bool crlf_yes = false, crlf_no = false, dosmode = false;
-
-// undos, only if each \n has \r preceeding.
-	if (iuConvert) {
-		for (i = 0; i < *partSize_p; ++i) {
-			if (rbuf[i] != '\n') continue;
-			if (i && rbuf[i - 1] == '\r') crlf_yes = true;
-			else crlf_no = true;
-		}
-		if (crlf_yes && !crlf_no) dosmode = true;
-	} // iuConvert
-
-	if (dosmode) {
-		if ((debugLevel >= 2 || (debugLevel == 1 && showMessage))
-		&& firstPart)
-			i_puts(MSG_ConvUnix);
-		for (i = j = 0; i < *partSize_p; ++i) {
-			char c = rbuf[i];
-			if (c == '\r' && rbuf[i + 1] == '\n')
-				continue;
-			rbuf[j++] = c;
-		}
-		rbuf[j] = 0;
-		fileSize -= (*partSize_p - j);
-		*partSize_p = j;
-	}
-
-	if (!iuConvert) return;
-
-/* Classify this incoming text as ascii or 8859 or utf-x */
 	bool is8859 = false, isutf8 = false;
 	int bom = 0;
+// Classify this incoming text as ascii or 8859 or utf-x
 	if(firstPart) bom = byteOrderMark((uchar *) rbuf, (int)fileSize);
+
 	if (bom) {
 // bom implies not reading by parts, so don't worry about that any more.
 		debugPrint(3, "text type is %s%s",
@@ -2063,15 +2040,42 @@ void diagnoseAndConvert (char **rbuf_p, bool *isAllocated_p, int *partSize_p, co
 			fileSize += (*partSize_p - oldSize);
 		}
 		if (cons_utf8 && isutf8 && firstPart) {
-// Strip off the leading bom, if any, and no we're not going to put it back.
+// Strip off the leading bom, if any.
 			if (fileSize >= 3 &&
 			    !memcmp(rbuf, "\xef\xbb\xbf", 3)) {
+				if ((debugLevel >= 2 || (debugLevel == 1 && showMessage)))
+					i_puts(MSG_RemovingBOM);
 				fileSize -= 3, *partSize_p -= 3;
 				memmove(rbuf, rbuf + 3, *partSize_p);
+				if (!cw->dol)
+					cw->utf8Mark = true;
 			}
 		}
 	}
 
+// undos, only if each \n has \r preceeding.
+	for (i = 0; i < *partSize_p; ++i) {
+		if (rbuf[i] != '\n') continue;
+		if (i && rbuf[i - 1] == '\r') crlf_yes = true;
+		else crlf_no = true;
+	}
+	if (crlf_yes && !crlf_no) dosmode = true;
+
+	if (dosmode) {
+		if ((debugLevel >= 2 || (debugLevel == 1 && showMessage))
+		&& firstPart)
+			i_puts(MSG_ConvUnix);
+		for (i = j = 0; i < *partSize_p; ++i) {
+			char c = rbuf[i];
+			if (c == '\r' && rbuf[i + 1] == '\n')
+				continue;
+			rbuf[j++] = c;
+		}
+		rbuf[j] = 0;
+		fileSize -= (*partSize_p - j);
+		*partSize_p = j;
+	}
+
 // if reading into an empty buffer, set the mode and print message
 	if (!cw->dol) {
 		if (bom & 2) {
@@ -2225,7 +2229,7 @@ what was successfully decoded.
 int base64Decode(char *start, char **end)
 {
 	char *b64_end = *end;
-	uchar val, leftover, mod;
+	uchar val, leftover = 0, mod;
 	bool equals;
 	int ret = GOOD_BASE64_DECODE;
 	char c, *q, *r;
@@ -2533,7 +2537,7 @@ char *closeColor(const char *s)
 		"yellowgreen", 0x9a, 0xcd, 0x32}, {
 		0}
 	};
-	const struct reserved *c, *best_c;
+	const struct reserved *c, *best_c = 0;
 	int best_val;
 	int r1, g1, b1;
 	const char *t;
@@ -3016,7 +3020,7 @@ top:
 
 // Messages in your host language.
 static const char **messageArray;
-char eb_language[9];
+char eb_language[8];
 int eb_lang;
 const char *const supported_languages[] = { 0,
 	"english", "french", "portuguese", "polish",
@@ -3074,8 +3078,9 @@ According to the rfc standards. So LC_TIME needs to be C.
 	eb_language[7] = 0;
 	caseShift(eb_language, 'l');
 	dot = strchr(eb_language, '.');
-	if (dot)
-		*dot = 0;
+	if (dot) *dot = 0;
+	for(s = eb_language; *s; ++s)
+		if(*s == '_') *s = '-';
 
 	if (!strncmp(eb_language, "en", 2))
 		return;		// english is already default
@@ -3314,3 +3319,225 @@ bool helpUtility(void)
 	return true;
 }
 
+// An edbrowse variable, primarily for scripting
+static struct EBVAR {
+	struct EBVAR *next;
+	const char *name, *value;
+	} *varhead;
+
+static const char *varLookup(const char *name)
+{
+	const struct EBVAR *v;
+	for(v = varhead; v; v = v->next)
+		if(stringEqual(name, v->name)) return v->value;
+	return NULL;
+}
+
+static void varSet(const char *name, const char *value)
+{
+	struct EBVAR *v, *v1 = 0;
+	bool isclear = stringEqual(value, "clear");
+	for(v = varhead; v; v1 = v, v = v->next)
+		if(stringEqual(name, v->name)) break;
+	if(v) {
+		cnzFree(v->value);
+		if(!isclear) {
+			v->value = cloneString(value);
+			return;
+		}
+// clear this variable
+		cnzFree(v->name);
+		if(v1) v1->next = v->next;
+		else varhead = v->next;
+		nzFree(v);
+		return;
+	}
+	if(isclear) return;
+	v = allocMem(sizeof(struct EBVAR));
+	v->next = varhead, varhead = v;
+	v->name = cloneString(name);
+	v->value = cloneString(value);
+}
+
+bool varCommand(const char *line)
+{
+	const char *v;
+	char cut;
+// I will cast to char * to blank out a character; but I'll put it back.
+	char *t;
+	skipWhite(&line);
+	t = (char*)line;
+	if(!isalpha(*t)) goto fail;
+	while(isalpha(*t) || isdigit(*t)) ++t;
+	cut = *t;
+	if(cut != '=' && cut != ':') goto fail;
+	v = t + 1;
+	if(cut == ':') {
+// set environment variable
+		if(*v != '=') goto fail;
+		++v;
+		*t = 0;
+		if(stringEqual(v, "clear"))
+			unsetenv(line);
+		else
+			setenv(line, v, 1);
+		*t = cut;
+		return true;
+	}
+	*t = 0;
+	varSet(line, v);
+	*t = cut;
+	return true;
+
+fail:
+	setError(MSG_VarSyntax);
+	return false;
+}
+
+// expand variables, result is allocated as it could be considerably longer
+bool varExpand(const char *line, char **newline)
+{
+	*newline = 0;
+// high runner case first
+	if(!strstr(line, "$(")) return true;
+
+// ok folks, there is something to look at
+	char *ns; // new string
+	int ns_l; // length of new string
+	ns = initString(&ns_l);
+	const char *l1 = line, *l2, *l3, *lv;
+	char *t; // I may cast to char*, to blank out something, but I'll put it back
+	char cut, op1, op2;
+	const char *value;
+	int total, side, n = 0, sign;
+
+top:
+	l2 = strstr(l1, "$(");
+	if(!l2) { // done
+		stringAndString(&ns, &ns_l, l1);
+		if(stringEqual(ns, line)) {
+			nzFree(ns);
+			return true;
+		}
+		*newline = ns;
+		return true;
+	}
+
+	if(l2 > l1) stringAndBytes(&ns, &ns_l, l1, l2 - l1);
+	l1 = l2;
+	l3 = (l2 += 2);
+	if(*l3 == ':') ++l3;
+	lv = l3;
+	if(!isalpha(*l3)) {
+syntax:
+// doesn't look right, just step ahead.
+		stringAndString(&ns, &ns_l, "$(");
+		l1 += 2;
+		goto top;
+	}
+	while(isalnum(*l3)) ++l3;
+	cut = *l3;
+	if(!cut) goto syntax;
+	if(!strchr(")+-*/", cut)) goto syntax;
+	t = (char*)l3, *t = 0;
+	if(*l2 == ':') value = getenv(lv);
+	else value = varLookup(lv);
+	if(!value) {
+reference:
+		nzFree(ns);
+		setError(MSG_NoEbVar, lv);
+		*t = cut;
+		return false;
+	}
+
+	if(cut == ')') { // simple variable reference
+		*t = cut;
+		stringAndString(&ns, &ns_l, value);
+		l1 = l3 + 1;
+		goto top;
+	}
+
+// an arithmetic expression begins here, every value henceforth
+// has to be a number
+	sign = 1;
+	if(*value == '+') side = stringIsNum(value + 1);
+	else if(*value == '-') sign = -1, side = stringIsNum(value + 1);
+	else side = stringIsNum(value);
+	if(side < 0) {
+notnumber:
+		nzFree(ns);
+		setError(MSG_VarNumber, lv);
+		*t = cut;
+		return false;
+	}
+	side *= sign;
+	total = 0, op1 = '+', op2 = cut;
+	*t = cut;
+
+nextarg:
+	l2 = ++l3;
+	if(*l3 == ':') ++l3;
+	lv = l3;
+	if(isdigit(*l3)) {
+		n = strtol(l3, (char**)&l3, 10);
+	} else {
+		if(!isalpha(*l3)) goto syntax;
+		while(isalnum(*l3)) ++l3;
+	}
+	cut = *l3;
+	if(!cut) goto syntax;
+	if(!strchr(")+-*/", cut)) goto syntax;
+	t = (char*)l3, *t = 0;
+	if(isalpha(*lv)) {
+		if(*l2 == ':') value = getenv(lv);
+		else value = varLookup(lv);
+		if(!value) goto reference;
+		sign = 1;
+		if(*value == '+') n = stringIsNum(value + 1);
+		else if(*value == '-') sign = -1, n = stringIsNum(value + 1);
+		else n = stringIsNum(value);
+		if(n < 0) goto notnumber;
+		n *= sign;
+	}
+
+// total op1 side op2 n cut
+// op1 is always + or -
+	if(op2 == '*') {
+		side *= n;
+	}
+
+	if(op2 == '/') {
+		if(!n) {
+			nzFree(ns);
+			setError(MSG_Div0);
+			*t = cut;
+			return false;
+		}
+// simulate $((x/y)) in the shell
+// there's no floating point here
+		int side1 = side, n1 = n;
+		if(side1 < 0) side1 = -side1;
+		if(n1 < 0) n1 = -n1;
+		side1 /= n1;
+		if(side < 0) side1 = -side1;
+		if(n < 0) side1 = -side1;
+		side = side1;
+	}
+
+	if(op2 != '*' && op2 != '/') {
+		if(op1 == '+') total += side; else total -= side;
+		side = n;
+		op1 = op2;
+	}
+
+	*t = cut;
+	if(cut == ')') { // end of expression
+		if(op1 == '+') total += side; else total -= side;
+		stringAndNum(&ns, &ns_l, total);
+		l1 = l3 + 1;
+		goto top;
+	}
+	op2 = cut;
+	goto nextarg;
+}
+
