@@ -6,6 +6,7 @@
 #include <time.h>
 
 bool curlActive;
+const char *curlCiphers;
 int redirect_count = 0;
 char *serverData;
 int serverDataLen;
@@ -13,7 +14,7 @@ CURL *global_http_handle;
 CURLSH *global_share_handle;
 bool pluginsOn = true;
 bool down_bg;			// download in background
-bool down_jsbg;		// download js in background
+bool down_jsbg = true;		// download js in background
 char showProgress = 'd';	// dots
 static char *httpLanguage;	/* outgoing */
 
@@ -56,17 +57,16 @@ void eb_curl_global_init(void)
 	const unsigned int least_acceptable_version =
 	    (major << 16) | (minor << 8) | patch;
 	curl_version_info_data *version_data = NULL;
+
 	CURLcode curl_init_status = curl_global_init(CURL_GLOBAL_ALL);
-	if (curl_init_status != 0)
-		goto libcurl_init_fail;
+	if (curl_init_status != 0) goto libcurl_init_fail;
 	version_data = curl_version_info(CURLVERSION_NOW);
 	if (version_data->version_num < least_acceptable_version)
 		i_printfExit(MSG_CurlVersion, major, minor, patch);
 
 // Initialize the global handle, to manage the cookie space.
 	global_share_handle = curl_share_init();
-	if (global_share_handle == NULL)
-		goto libcurl_init_fail;
+	if (global_share_handle == NULL) goto libcurl_init_fail;
 
 	curl_share_setopt(global_share_handle, CURLSHOPT_LOCKFUNC, lock_share);
 	curl_share_setopt(global_share_handle, CURLSHOPT_UNLOCKFUNC,
@@ -79,31 +79,25 @@ void eb_curl_global_init(void)
 			  CURL_LOCK_DATA_SSL_SESSION);
 
 	global_http_handle = curl_easy_init();
-	if (global_http_handle == NULL)
-		goto libcurl_init_fail;
+	if (global_http_handle == NULL) goto libcurl_init_fail;
 	if (cookieFile && !ismc) {
 		curl_init_status =
 		    curl_easy_setopt(global_http_handle, CURLOPT_COOKIEFILE,
 				     "");
-		if (curl_init_status != CURLE_OK) {
-			goto libcurl_init_fail;
-		}
+		if (curl_init_status != CURLE_OK) goto libcurl_init_fail;
 		curl_init_status =
 		    curl_easy_setopt(global_http_handle, CURLOPT_COOKIEJAR,
 				     cookieFile);
-		if (curl_init_status != CURLE_OK)
-			goto libcurl_init_fail;
+		if (curl_init_status != CURLE_OK) goto libcurl_init_fail;
 	}
 	curl_init_status =
 	    curl_easy_setopt(global_http_handle, CURLOPT_ENCODING, "");
-	if (curl_init_status != CURLE_OK)
-		goto libcurl_init_fail;
+	if (curl_init_status != CURLE_OK) goto libcurl_init_fail;
 
 	curl_init_status =
 	    curl_easy_setopt(global_http_handle, CURLOPT_SHARE,
 			     global_share_handle);
-	if (curl_init_status != CURLE_OK)
-		goto libcurl_init_fail;
+	if (curl_init_status != CURLE_OK) goto libcurl_init_fail;
 	curlActive = true;
 	return;
 
@@ -391,11 +385,11 @@ eb_curl_callback(char *incoming, size_t size, size_t nitems, struct i_get * g)
 			goto showdots;
 		}
 		if (g->down_state == 2) {
-// has to be the foreground http thread, so ok to call setErro,
+// has to be the foreground http thread, so ok to call setError,
 // which is not threadsafe.
-			setError(MSG_NoWrite2, g->down_file);
+			setError(MSG_NoWrite2, g->down_file, strerror(errno));
 		} else {
-			i_printf(MSG_NoWrite2, g->down_file);
+			i_printf(MSG_NoWrite2, g->down_file, strerror(errno));
 			printf(", ");
 			i_puts(MSG_DownAbort);
 		}
@@ -772,7 +766,7 @@ static void urlSanitize(struct i_get *g, const char *post)
 
 	if (g->uriEncoded && !looksPercented(url, post)) {
 // this test is limited to characters that really really shouldn't be there.
-		debugPrint(2, "Warning, url %s doesn't look encoded", url);
+		debugPrint(3, "Warning, url %s doesn't look encoded", url);
 		g->uriEncoded = false;
 	}
 
@@ -1721,7 +1715,7 @@ static void ftp_listing(struct i_get *g)
 /* Format a line from a gopher directory. */
 static void gopher_ls_line(struct i_get *g, char *line)
 {
-	int port;
+	int port = 0;
 	char first, *text, *pathname, *host, *s, *plus;
 	int l = strlen(line);
 	if (l && line[l - 1] == '\r')
@@ -2160,7 +2154,7 @@ We don't need to do that either.
 
 	f = fopen(tempfile, (cw->binMode ? "rb" : "r"));
 	if(!f) { // this should never happen
-		setError(MSG_NoOpen, tempfile);
+		setError(MSG_NoOpen, tempfile, strerror(errno));
 		goto fail;
 	}
 	curl_easy_setopt(h, CURLOPT_UPLOAD, 1L);
@@ -2353,8 +2347,10 @@ static bool dataConnect(struct i_get *g)
 	if(!comma) { // should never happen
 		debugPrint(3, "data has no comma");
 		nzFree(copy);
+		g->code = 403;
 		return false;
 	}
+	g->code = 200;
 	*comma++ = 0;
 	unpercentString(comma);
 	if(comma - copy >= 8 && stringEqual(comma - 8, ";base64")) {
@@ -2385,8 +2381,8 @@ static bool dataConnect(struct i_get *g)
 	g->hcl = g->length;
 	debugPrint(4, "content length %lld", g->hcl);
 	nzFree(copy);
-	g->cfn = cloneString("data:");
 	g->cfn = suggestedDownloadName(g->gotag);
+	if(!g->cfn) g->cfn = cloneString("data:");
 	return true;
 }
 
@@ -2489,13 +2485,12 @@ static CURL *http_curl_init(struct i_get *g)
 #endif
 	curl_easy_setopt(h, CURLOPT_HTTPAUTH, curl_auth);
 
-#if 0
-// in case you run into DH key too small
-// This may not be portable, e.g. curl compiled with gnutls;
-// though it is usually compiled with openssl.
-// Not sure of the best solution here.
-	curl_easy_setopt(h, CURLOPT_SSL_CIPHER_LIST, "DEFAULT@SECLEVEL=1");
-#endif
+// If you run into DH key too small, set SECLEVEL = 1.
+// However, this only works for openssl.
+// Without this line, you are at the whim of /etc/ssl/openssl.cnf, which might
+// set a smaller key size, which most servers don't care about, but some do.
+	if(curlCiphers)
+		curl_easy_setopt(h, CURLOPT_SSL_CIPHER_LIST, curlCiphers);
 
 /* The next few setopt calls could allocate or perform file I/O. */
 	g->error[0] = '\0';
@@ -2884,11 +2879,11 @@ got_answer:
 	g->down_fd = creat(answer, MODE_rw);
 	if (g->down_fd < 0) {
 		if(!preset) {
-			i_printf(MSG_NoCreate2, answer);
+			i_printf(MSG_NoCreate2, answer, strerror(errno));
 			nl();
 			goto top;
 		}
-		setError(MSG_NoCreate2, answer);
+		setError(MSG_NoCreate2, answer, strerror(errno));
 		nzFree(fp2), nzFree(down_prefile), down_prefile = 0;
 		g->down_state = -1;
 		return;
@@ -3153,7 +3148,7 @@ int frameExpandLine(int ln, Tag *t)
 		s = stringInBufLine((char *)line, "Frame ");
 		if (!s)
 			return 1;
-		if ((s = strchr(s, InternalCodeChar)) == NULL)
+		if ((s = charInBufLine(s, InternalCodeChar)) == NULL)
 			return 2;
 		tagno = strtol(s + 1, (char **)&s, 10);
 		if (tagno < 0 || tagno >= cw->numTags || *s != '{')
