@@ -2,7 +2,6 @@
 
 #include "eb.h"
 
-#include <pthread.h>
 #include <signal.h>
 
 // Define the globals that are declared in eb.h.
@@ -11,10 +10,9 @@
 Window *cw;
 Frame *cf;
 int gfsn; // global frame sequence number
-int ignore;
 const char *progname;
 const char eol[] = "\r\n";
-const char *version = "3.8.9";
+const char *version = "3.8.10";
 char *changeFileName;
 char *configFile, *addressFile, *cookieFile, *emojiFile;
 char *mailDir, *mailUnread, *mailStash, *mailReply;
@@ -379,7 +377,7 @@ const char *mailRedirect(const char *to, const char *from,
 	unsigned i;
 	int k;
 	struct ebhost *f = ebhosts;
-	const char *m, *r;	// match and redirect
+	const char *m = 0, *r = 0;	// match and redirect
 
 	for (i = 0; i < ebhosts_avail; ++i, ++f) {
 		char type = f->type;
@@ -488,6 +486,11 @@ int main(int argc, char **argv)
 	selectLanguage();
 	setHTTPLanguage(eb_language);
 
+	curlCiphers = getenv("EBCIPHERS");
+	if(curlCiphers && !*curlCiphers) curlCiphers = 0;
+// 1 is shorthand for the only seting we know of so far
+	if(curlCiphers && stringEqual(curlCiphers, "1")) curlCiphers = "DEFAULT@SECLEVEL=1";
+
 // Establish the home directory, and standard edbrowse files thereunder.
 	home = getenv("HOME");
 
@@ -1210,7 +1213,6 @@ ahead:
 			if (now >= cw->nextrender)
 				rerender(-1);
 		}
-		jClearSync();
 		ok = edbrowseCommand(new, true);
 		free(new);
 
