@@ -329,7 +329,7 @@ package:
 
 /*********************************************************************
 Return a string that defines the charset of the outgoing mail.
-This just looks at your language setting - reaaly dumb.
+This just looks at your language setting - really dumb.
 It should interrogate each file/attachment.
 Well, this will get us started.
 *********************************************************************/
@@ -362,7 +362,7 @@ bool *long_p)
 {
 	char *buf;
 	char c;
-	bool longline, longword, cr, endlinespace, startlinedot, flowed;
+	bool longline, longword, cr, endlinespace, startlinedot, flowed = false;
 	char *s, *t, *v;
 	char *ct, *ce;		// content type, content encoding
 	int buflen, i, cx;
@@ -894,6 +894,8 @@ static struct curl_slist *buildRecipientSList(const char **recipients)
 	return recipient_slist;
 }
 
+static char cerror[CURL_ERROR_SIZE + 1];
+
 static CURL *newSendmailHandle(const struct MACCOUNT *account,
 			       const char *outurl, const char *reply,
 			       struct curl_slist *recipients)
@@ -906,6 +908,9 @@ static CURL *newSendmailHandle(const struct MACCOUNT *account,
 		return NULL;
 	}
 
+	if(curlCiphers)
+		curl_easy_setopt(handle, CURLOPT_SSL_CIPHER_LIST, curlCiphers);
+	curl_easy_setopt(handle, CURLOPT_ERRORBUFFER, cerror);
 	curl_easy_setopt(handle, CURLOPT_CONNECTTIMEOUT, mailTimeout);
 	res = setCurlURL(handle, outurl);
 	if (res != CURLE_OK) {
@@ -947,7 +952,7 @@ static CURL *newSendmailHandle(const struct MACCOUNT *account,
 
 new_handle_cleanup:
 	if (res != CURLE_OK) {
-		ebcurl_setError(res, outurl, 0, emptyString);
+		ebcurl_setError(res, outurl, 0, cerror);
 		curl_easy_cleanup(handle);
 		handle = NULL;
 	}
@@ -988,6 +993,7 @@ sendMailSMTP(const struct MACCOUNT *account, const char *reply,
 		debugPrint(6, "%s", message);
 		puts("debug don't send");
 	} else {
+		cerror[0] = 0;
 		res = curl_easy_perform(handle);
 	}
 	if (res == CURLE_OK)
@@ -995,7 +1001,7 @@ sendMailSMTP(const struct MACCOUNT *account, const char *reply,
 
 smtp_cleanup:
 	if (res != CURLE_OK)
-		ebcurl_setError(res, smtp_url, 0, emptyString);
+		ebcurl_setError(res, smtp_url, 0, cerror);
 	if (handle)
 		curl_easy_cleanup(handle);
 	curl_slist_free_all(recipient_slist);
@@ -1196,8 +1202,12 @@ sendMail(int account, const char **recipients, const char *body,
 		stringAndBytes(&out, &j, refline, s - refline);
 		stringAndString(&out, &j, eol);
 	}
+// why should we tell them who we are?
+// I've never received an email that contained User-Agent
+#if 0
 	sprintf(serverLine, "User-Agent: %s%s", currentAgent, eol);
 	stringAndString(&out, &j, serverLine);
+#endif
 	if (subjectLine[0]) {
 		sprintf(serverLine, "Subject: %s%s", subjectLine, eol);
 		stringAndString(&out, &j, serverLine);
