@@ -21,6 +21,7 @@ bool gotimers = true;
 bool uvw;
 int timerspeed = 1;
 long long fileSize;
+int ignore;
 char *downDir, *down_prefile, *home;
 bool endMarks;		// ^ $ on listed lines
 uchar dirWrite;		// directories read write
@@ -451,11 +452,27 @@ bool stringEqualCI(const char *s, const char *t)
 
 // a text line in the buffer isn't a string; you can't use strstr.
 // It ends in \n
+// Stop as soon as the target string overlaps the end of s to avoid
+// peeking past the end of s
 const char *stringInBufLine(const char *s, const char *t)
 {
 	int n = strlen(t);
+	int i;
+	for (i = 0; i < n - 1; i++) {
+		if (s[i] == '\n') return 0; // too short
+	}
+	while(1) {
+		if (!strncmp(s, t, n)) return s;
+		if(s[n - 1] == '\n') break;
+		++s;
+	}
+	return 0;
+}
+
+const char *charInBufLine(const char *s, char t)
+{
 	for (; *s != '\n'; ++s) {
-		if (!strncmp(s, t, n))
+		if (*s == t)
 			return s;
 	}
 	return 0;
@@ -713,8 +730,9 @@ functionality, such as edpager.
 inpart = 0: read the whole file
 inpart = 1: read the first part
 inpart = 2: read the next part
+It is more efficient if PARTSIZE is a multiple of the page size.
 *********************************************************************/
-#define FILEPARTSIZE 10000000
+#define FILEPARTSIZE 0x4000000
 
 int fdIntoMemory(int fd, char **data, int *len, bool inparts)
 {
@@ -736,7 +754,9 @@ int fdIntoMemory(int fd, char **data, int *len, bool inparts)
 			nzFree(chunk);
 			*data = emptyString;
 			*len = 0;
+// fileIntoMemory improves on this error message
 			setError(MSG_BigFile);
+			errno = EFBIG;
 			return 0;
 		}
 		n = read(fd, chunk, blocksize);
@@ -745,7 +765,8 @@ int fdIntoMemory(int fd, char **data, int *len, bool inparts)
 			nzFree(chunk);
 			*data = emptyString;
 			*len = 0;
-			setError(MSG_NoRead, "file descriptor");
+// fileIntoMemory improves on this error message
+			setError(MSG_NoRead2, "file descriptor", strerror(errno));
 			return 0;
 		}
 
@@ -794,31 +815,29 @@ int fileIntoMemory(const char *filename, char **data, int *len, bool inparts)
 	}
 	fh = open(filename, O_RDONLY | O_BINARY);
 	if (fh < 0) {
-		setError(MSG_NoOpen, filename);
+		setError(MSG_NoOpen, filename, strerror(errno));
 		return 0;
 	}
 
 fh_set:
 	ret = fdIntoMemory(fh, data, len, inparts);
 	if (ret == 0 && filename)
-		setError(MSG_NoRead2, filename);
+		setError(MSG_NoRead2, filename, strerror(errno));
 	if(ret <= 1) close(fh);
 	return ret;
 }
 
 // inverse of the above
-bool memoryOutToFile(const char *filename, const char *data, int len,
-/* specify the error messages */
-		     int msgcreate, int msgwrite)
+bool memoryOutToFile(const char *filename, const char *data, int len)
 {
 	int fh =
 	    open(filename, O_CREAT | O_TRUNC | O_WRONLY | O_BINARY, MODE_rw);
 	if (fh < 0) {
-		setError(msgcreate, filename, errno);
+			setError(MSG_TempNoCreate2, filename, strerror(errno));
 		return false;
 	}
 	if (write(fh, data, len) < len) {
-		setError(msgwrite, filename, errno);
+			setError(MSG_NoWrite2, filename, strerror(errno));
 		close(fh);
 		return false;
 	}
@@ -830,9 +849,9 @@ bool memoryOutToFile(const char *filename, const char *data, int len,
 void truncate0(const char *filename, int fh)
 {
 	if (fh < 0)
-		truncate(filename, 0l);
+		ignore = truncate(filename, 0l);
 	else
-		ftruncate(fh, 0l);
+		ignore = ftruncate(fh, 0l);
 }
 
 long long bufferSizeW(const Window *w, bool browsing)
@@ -1363,7 +1382,7 @@ int getche(void)
 	if(!isInteractive) return getchar();
 	fflush(stdout);
 	ttyRaw(1, 0, true);
-	read(0, &c, 1);
+	ignore = read(0, &c, 1);
 	ttyRestoreSettings();
 	return c;
 }
@@ -1374,7 +1393,7 @@ int getch(void)
 	if(!isInteractive) return getchar();
 	fflush(stdout);
 	ttyRaw(1, 0, false);
-	read(0, &c, 1);
+	ignore = read(0, &c, 1);
 	ttyRestoreSettings();
 	return c;
 }
@@ -1632,7 +1651,7 @@ static int dircmp_st(const void *s, const void *t)
 {
 	const struct DSR *q = s;
 	const struct DSR *r = t;
-	int rc;
+	int rc = 0;
 	if (ls_sort == 1) {
 		rc = 0;
 		if (q->u.z < r->u.z)
@@ -1833,6 +1852,7 @@ bool readDirectory(const char *filename, int endline, char cmd, struct lineMap *
 	if(dmap) {
 		cw->dmap = allocMem((linecount + 1) * DTSIZE);
 		memcpy(cw->dmap + DTSIZE, dmap, linecount*DTSIZE);
+		free(dmap);
 	}
 
 success:
@@ -1899,7 +1919,7 @@ abort:
 				setError(MSG_MetaChar);
 				goto abort;
 			}
-			asprintf(&a, "rm -rf %c%s%c",
+			ignore = asprintf(&a, "rm -rf %c%s%c",
 			qc, path, qc);
 			j = system(a);
 			free(a);
@@ -1915,7 +1935,7 @@ abort:
 		if (dirWrite == 2 || (*ftype && strchr("@<*^|", *ftype))) {
 unlink:
 			if (unlink(path)) {
-				setError(MSG_NoRemove, file);
+				setError(MSG_NoRemove, file, strerror(errno));
 				goto abort;
 			}
 		} else {
@@ -1932,7 +1952,7 @@ unlink:
 							setError(MSG_MetaChar);
 							goto abort;
 						}
-						asprintf(&a, "mv -n %c%s%c %c%s%c",
+						ignore = asprintf(&a, "mv -n %c%s%c %c%s%c",
 						qc, path, qc, qc, bin, qc);
 						j = system(a);
 						free(a);
@@ -1946,9 +1966,7 @@ unlink:
 					}
 					if (!fileIntoMemory    (path, &rmbuf, &rmlen, 0))
 						goto abort;
-					if (!memoryOutToFile(bin, rmbuf, rmlen,
-							     MSG_TempNoCreate2,
-							     MSG_NoWrite2)) {
+					if (!memoryOutToFile(bin, rmbuf, rmlen)) {
 						nzFree(rmbuf);
 						goto abort;
 					}
@@ -1957,7 +1975,7 @@ unlink:
 				}
 
 // some other rename error
-				setError(MSG_NoMoveToTrash, file);
+				setError(MSG_NoMoveToTrash, file, strerror(errno));
 				goto abort;
 			}
 		}
@@ -1972,7 +1990,7 @@ unlink:
 // Move or copy files from one directory to another
 bool moveFiles(int start, int end, int dest, char origcmd, char relative)
 {
-	Window *cw1 = cw, *cw2, *w;
+	Window *cw1 = cw, *cw2 = 0, *w;
 	char *path1, *path2;
 	int ln, cnt, dol;
 
@@ -2060,6 +2078,17 @@ bool moveFiles(int start, int end, int dest, char origcmd, char relative)
 			return false;
 		}
 
+		if(tolower(origcmd) == 'l') {
+			if((origcmd == 'l' && link(path1, path2)) ||
+			(origcmd == 'L' && symlink(path1, path2))) {
+				setError(MSG_LinkError, file, strerror(errno));
+				free(file);
+			free(path1);
+				return false;
+			}
+			goto moved;
+		}
+
 		errno = EXDEV;
 		if (origcmd == 't' || rename(path1, path2)) {
 			if (errno == EXDEV) {
@@ -2082,7 +2111,7 @@ bool moveFiles(int start, int end, int dest, char origcmd, char relative)
 						free(path1);
 						return false;
 					}
-					asprintf(&a, "%s %c%s%c %c%s%c",
+					ignore = asprintf(&a, "%s %c%s%c %c%s%c",
 					(origcmd == 'm' ? "mv -n" : "cp -an"),
 					qc, path1, qc, qc, cw2->baseDirName, qc);
 					j = system(a);
@@ -2103,9 +2132,7 @@ bool moveFiles(int start, int end, int dest, char origcmd, char relative)
 					free(path1);
 					return false;
 				}
-				if (!memoryOutToFile(path2, rmbuf, rmlen,
-						     MSG_TempNoCreate2,
-						     MSG_NoWrite2)) {
+				if (!memoryOutToFile(path2, rmbuf, rmlen)) {
 					free(file);
 					free(path1);
 					nzFree(rmbuf);
@@ -2130,7 +2157,7 @@ bool moveFiles(int start, int end, int dest, char origcmd, char relative)
 				goto moved;
 			}
 
-			setError(MSG_MoveError, file, errno);
+			setError(MSG_MoveError, file, strerror(errno));
 			free(file);
 			free(path1);
 			return false;
@@ -2154,9 +2181,15 @@ moved:
 		else
 			cw->dmap = allocZeroMem(DTSIZE * (dol + 1));
 		memset(cw->dmap + DTSIZE*dol, 0, DTSIZE);
-		cw->dmap[DTSIZE*dol] = ftype[0];
-		if(ftype[0])
-		cw->dmap[DTSIZE*dol + 1] = ftype[1];
+		if(origcmd == 'L') { // creating symbolic link
+			cw->dmap[DTSIZE*dol] = '@';
+			if(ftype[0] && ftype[0] != '@')
+				cw->dmap[DTSIZE*dol + 1] = ftype[0];
+		} else {
+			cw->dmap[DTSIZE*dol] = ftype[0];
+			if(ftype[0])
+				cw->dmap[DTSIZE*dol + 1] = ftype[1];
+		}
 // if attributes were displayed in that directory - more work to do.
 // I just leave a space for them; I don't try to derive them.
 		if(cw->r_map) {
@@ -2165,7 +2198,7 @@ moved:
 			cw->r_map[dol].text = (uchar*)emptyString;
 		}
 		cw = cw1; // go back to original window
-		if(origcmd == 't') cw->dot = ln++;
+		if(origcmd != 'm') cw->dot = ln++;
 	}
 
 	return true;
@@ -2403,7 +2436,7 @@ FILE *efopen(const char *name, const char *mode)
 		return f;
 
 	if (*mode == 'r')
-		i_printfExit(MSG_NoOpen, name);
+		i_printfExit(MSG_NoOpen, name, strerror(errno));
 	else if (*mode == 'w' || *mode == 'a')
 		i_printfExit(MSG_CreateFail, name);
 	else
@@ -2420,7 +2453,7 @@ int eopen(const char *name, int mode, int perms)
 	if (mode & O_WRONLY)
 		i_printfExit(MSG_CreateFail, name);
 	else
-		i_printfExit(MSG_NoOpen, name);
+		i_printfExit(MSG_NoOpen, name, strerror(errno));
 	return -1;
 }
 
